import pytest
from schema import Schema, Regex
from zenroom import zencode_exec #, zenroom_exec


def test_zencode_call_random_array():
    contract =  """Given nothing
When I create the array of '64' random numbers modulo '100'
and I create the aggregation of array 'array'
Then print the 'array'
and print the 'aggregation'
"""
    res = zencode_exec(
        contract
    )
    out_regex = r'{\"aggregation\":\"\d{4}\",\"array\":\[(?:\"\d{1,3}\"\,)+\"\d{1,3}\"\]}'
    assert Regex(out_regex).validate(res.output)
    assert Schema({
        'aggregation': str,
        'array': [str]
    }).validate(res.result)

def test_extra():
    script="""Scenario 'ecdh': Create the keypair
Given I have a 'string' named 'keys'
Given I have a 'string' named 'data'
Given I have a 'string' named 'extra'
Then print data
"""
    res = zencode_exec(script, "", '{"keys": "keys"}', '{"data": "data"}', '{"extra": "extra"}', "")
    out = '{"data":"data","extra":"extra","keys":"keys"}\n'
    assert res.output == out

def test_zencode_failure():
    contract = """
Given I have a 'string' named 'string'
Then print the data
"""
    res = zencode_exec(contract)
    assert("ERROR" in res.logs)

# def test_lua_call_hello_world():
#     lua_res = zenroom_exec(
#         "print('hello world')"
#     )
#     assert lua_res.output == 'hello world'



"""Zenroom bindings for python 3.6+"""

import json
from dataclasses import dataclass, field
import subprocess
import base64


@dataclass
class ZenResult():
    output: str = field()
    logs: str = field()

    def __post_init__(self):
        try:
            self.result = json.loads(self.output)
        except json.JSONDecodeError:
            self.result = None


def zencode_exec(script, conf=None, keys=None, data=None, extra=None, context=None):
    zen_input = []
    if conf:
        zen_input.append(conf)
    zen_input.append(b'\n')
    zen_input.append(base64.b64encode(script.encode()))
    zen_input.append(b'\n')
    if keys:
        zen_input.append(base64.b64encode(keys.encode()))
    zen_input.append(b'\n')
    if data:
        zen_input.append(base64.b64encode(data.encode()))
    zen_input.append(b'\n')
    if extra:
        zen_input.append(base64.b64encode(extra.encode()))
    zen_input.append(b'\n')
    if context:
        zen_input.append(base64.b64encode(context.encode()))
    zen_input.append(b'\n')
    res = subprocess.run(["zencode-exec"],
                         capture_output=True,
                         input=b''.join(zen_input))

    return ZenResult(res.stdout.decode(), res.stderr.decode())



import os
from setuptools import Extension, setup

PYTHON_ROOT = os.getcwd()
ZENROOM_ROOT = os.path.join(os.getcwd(), 'src')

def get_versions():
    with open(os.path.join(ZENROOM_ROOT, 'git_utils')) as f:
        zenroom_version = f.readline().strip('\n')
        python_version = f.readline().strip('\n')
    return python_version, zenroom_version


os.chdir(ZENROOM_ROOT)
python_version, zenroom_version = get_versions()
os.chdir(PYTHON_ROOT)


def get_readme():
    try:
        with open(os.path.join(ZENROOM_ROOT, 'docs/pages/python.md')) as f:
            return f.read()
    except IOError:
        pass


setup(
    name='zenroom',
    description='Zenroom for Python: Bindings of Zenroom library for Python.',
    version=python_version,
    long_description=get_readme(),
    long_description_content_type='text/markdown',
    license='AGPLv3',
    keywords='zenroom crypto-language-processing virtual-machine blockchain crypto ecc dyne ecdh ecdsa zero-knowledge-proofs javascript npm ecp2 miller-loop hamming-distance elgamal aes-gcm aead seccomp goldilocks'.split(),
    url='https://github.com/dyne/Zenroom',
    author='Danilo Spinella, David Dashyan, Puria Nafisi Azizi',
    author_email='danyspin@dyne.org, mail@davie.li, puria@dyne.org',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Security',
    ],
    project_urls={
        'Homepage': 'https://zenroom.org',
        'Source Code': 'https://github.com/dyne/Zenroom',
        'Documentation': 'https://dev.zenroom.org/',
    },
    packages=['zenroom'],
    include_package_data=True,
    python_requires='>=3.8, <4',
    extras_require={
        'dev': [],
        'test': ['pytest', 'schema'],
    },
)
